#include "Heap.h"
#include <stdio.h>
#include <assert.h>
#include <memory.h>
#include <stdlib.h>

void Serial();
void Random();
void Align ();
void Overwrite();

// Test Heap.cc
void main()
{
    printf("Hello, world\n");
    int nHeapSize = 1000000;
    char *pc = reinterpret_cast<char *>(malloc(nHeapSize));
    assert(pc);

    Init(pc, nHeapSize);

    Check();
    Stats();
    Dump ();

    Serial();
    Random();

    Check();
    Stats();
    Dump ();

    Align();

//  Overwrite();
}


// Allocate and free 100 blocks in a row
void Serial()
{
    int nBlocks = 100;
    char *apc[nBlocks];

    for (int i=0; i<nBlocks; i++)
    {
	apc[i] = Malloc(100);
	assert(apc[i]);
	memset(apc[i], 0, 100);
    }

    Stats();
    Check();

    for (int i=0; i<nBlocks; i++)
    {
	Free(apc[i]);
    }
}


// Allocate and free 1000 blocks at random
void Random()
{
    const int nBlocks = 1000;
    char *apc[nBlocks];
    int top = 0;

    for (int i=0; i<nBlocks; i++)
	apc[i] = 0;

    for (int i=0; i<nBlocks; i++)
    {
	int   nBlockSize = rand();
	char *pc = Malloc(nBlockSize);

	if (pc)
	{
	    memset(pc, 0, nBlockSize);
	    apc[top++] = pc;
	}
	else
	{
	    int n = rand() % nBlocks;
	    if (apc[n])
	    {
		Free(apc[n]);
		apc[n] = 0;
	    }
	}
    }

    Stats();
    Check();

    for (int i=0; i<nBlocks; i++)
	if (apc[i])
	    Free(apc[i]);
}


// Allocate two blocks and print their addresses
// They should be aligned on kAlign byte boundaries
void Align()
{
    char *pc1 = Malloc(1);
    printf("%p\n", pc1);

    char *pc2 = Malloc(1);
    printf("%p\n", pc2);

    Free(pc1);
    Free(pc2);
}


// Overwrite the boundaries of a block
// Free() should assert()
void Overwrite()
{
    char *pc = Malloc(100);
    pc[ -1] = 0;
    pc[100] = 0;
    Free(pc);
}

